/* Generated by re2c */
// re2c $INPUT -o $OUTPUT -igcd --flex-syntax --case-inverted
/*
   +----------------------------------------------------------------------+
   | Zend Engine                                                          |
   +----------------------------------------------------------------------+
   | Copyright (c) 1998-2015 Zend Technologies Ltd. (http://www.zend.com) |
   +----------------------------------------------------------------------+
   | This source file is subject to version 2.00 of the Zend license,     |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
   | http://www.zend.com/license/2_00.txt.                                |
   | If you did not receive a copy of the Zend license and are unable to  |
   | obtain it through the world-wide-web, please send a note to          |
   | license@zend.com so we can mail you a copy immediately.              |
   +----------------------------------------------------------------------+
   | Authors: Zeev Suraski <zeev@zend.com>                                |
   |          Jani Taskinen <jani@php.net>                                |
   |          Marcus Boerger <helly@php.net>                              |
   |          Nuno Lopes <nlopess@php.net>                                |
   |          Scott MacVicar <scottmac@php.net>                           |
   +----------------------------------------------------------------------+
*/

/* $Id$ */

#include <errno.h>
#include "zend.h"
#include "zend_API.h"
#include "zend_globals.h"
#include <zend_ini_parser.h>
#include "zend_ini_scanner.h"

#ifdef YYDEBUG
#undef YYDEBUG
#endif

#if 0
# define YYDEBUG(s, c) printf("state: %d char: %c\n", s, c)
#else
# define YYDEBUG(s, c)
#endif

#include "zend_ini_scanner_defs.h"

#define YYCTYPE   unsigned char
/* allow the scanner to read one null byte after the end of the string (from ZEND_MMAP_AHEAD)
 * so that if will be able to terminate to match the current token (e.g. non-enclosed string) */
#define YYFILL(n) { if (YYCURSOR > YYLIMIT) return 0; }
#define YYCURSOR  SCNG(yy_cursor)
#define YYLIMIT   SCNG(yy_limit)
#define YYMARKER  SCNG(yy_marker)

#define YYGETCONDITION()  SCNG(yy_state)
#define YYSETCONDITION(s) SCNG(yy_state) = s

#define STATE(name)  yyc##name

/* emulate flex constructs */
#define BEGIN(state) YYSETCONDITION(STATE(state))
#define YYSTATE      YYGETCONDITION()
#define yytext       ((char*)SCNG(yy_text))
#define yyleng       SCNG(yy_leng)
#define yyless(x)    do {	YYCURSOR = (unsigned char*)yytext + x; \
							yyleng   = (unsigned int)x; } while(0)

/* #define yymore()     goto yymore_restart */

/* perform sanity check. If this message is triggered you should
   increase the ZEND_MMAP_AHEAD value in the zend_streams.h file */
#define YYMAXFILL 6

#if ZEND_MMAP_AHEAD < (YYMAXFILL + 1)
# error ZEND_MMAP_AHEAD should be greater than YYMAXFILL
#endif


/* How it works (for the core ini directives):
 * ===========================================
 *
 * 1. Scanner scans file for tokens and passes them to parser.
 * 2. Parser parses the tokens and passes the name/value pairs to the callback
 *    function which stores them in the configuration hash table.
 * 3. Later REGISTER_INI_ENTRIES() is called which triggers the actual
 *    registering of ini entries and uses zend_get_configuration_directive()
 *    to fetch the previously stored name/value pair from configuration hash table
 *    and registers the static ini entries which match the name to the value
 *    into EG(ini_directives) hash table.
 * 4. PATH section entries are used per-request from down to top, each overriding
 *    previous if one exists. zend_alter_ini_entry() is called for each entry.
 *    Settings in PATH section are ZEND_INI_SYSTEM accessible and thus mimics the
 *    php_admin_* directives used within Apache httpd.conf when PHP is compiled as
 *    module for Apache.
 * 5. User defined ini files (like .htaccess for apache) are parsed for each request and
 *    stored in separate hash defined by SAPI.
 */

/* TODO: (ordered by importance :-)
 * ===============================================================================
 *
 *  - Separate constant lookup totally from plain strings (using CONSTANT pattern)
 *  - Add #if .. #else .. #endif and ==, !=, <, > , <=, >= operators
 *  - Add #include "some.ini"
 *  - Allow variables to refer to options also when using parse_ini_file()
 *
 */

/* Globals Macros */
#define SCNG	INI_SCNG
#ifdef ZTS
ZEND_API ts_rsrc_id ini_scanner_globals_id;
#else
ZEND_API zend_ini_scanner_globals ini_scanner_globals;
#endif

/* Eat leading whitespace */
#define EAT_LEADING_WHITESPACE()                     \
	while (yytext[0]) {                              \
		if (yytext[0] == ' ' || yytext[0] == '\t') { \
			SCNG(yy_text)++;                         \
			yyleng--;                                \
		} else {                                     \
			break;                                   \
		}                                            \
	}

/* Eat trailing whitespace + extra char */
#define EAT_TRAILING_WHITESPACE_EX(ch)              \
	while (yyleng > 0 && (                          \
		(ch != 'X' && yytext[yyleng - 1] ==  ch) || \
		yytext[yyleng - 1] == '\n' ||               \
		yytext[yyleng - 1] == '\r' ||               \
		yytext[yyleng - 1] == '\t' ||               \
		yytext[yyleng - 1] == ' ')                  \
	) {                                             \
		yyleng--;                                   \
	}

/* Eat trailing whitespace */
#define EAT_TRAILING_WHITESPACE()	EAT_TRAILING_WHITESPACE_EX('X')

#define zend_ini_copy_value(retval, str, len)	\
	ZVAL_NEW_STR(retval, zend_string_init(str, len, 1))


#define RETURN_TOKEN(type, str, len) {                       \
	if (SCNG(scanner_mode) == ZEND_INI_SCANNER_TYPED) {      \
		zend_ini_copy_typed_value(ini_lval, type, str, len); \
	} else {                                                 \
		zend_ini_copy_value(ini_lval, str, len);             \
	}                                                        \
	return type;                                             \
}

static inline int convert_to_number(zval *retval, const char *str, const int str_len)
{
	zend_uchar type;
	int overflow;
	zend_long lval;
	double dval;

	if ((type = is_numeric_string_ex(str, str_len, &lval, &dval, 0, &overflow)) != 0) {
		if (type == IS_LONG) {
			ZVAL_LONG(retval, lval);
			return SUCCESS;
		} else if (type == IS_DOUBLE && !overflow) {
			ZVAL_DOUBLE(retval, dval);
			return SUCCESS;
		}
	}

	return FAILURE;
}

static void zend_ini_copy_typed_value(zval *retval, const int type, const char *str, int len)
{
	switch (type) {
		case BOOL_FALSE:
		case BOOL_TRUE:
			ZVAL_BOOL(retval, type == BOOL_TRUE);
			break;

		case NULL_NULL:
			ZVAL_NULL(retval);
			break;

		case TC_NUMBER:
			if (convert_to_number(retval, str, len) == SUCCESS) {
				break;
			}
			/* intentional fall-through */
		default:
			zend_ini_copy_value(retval, str, len);
	}
}

static void _yy_push_state(int new_state)
{
	zend_stack_push(&SCNG(state_stack), (void *) &YYGETCONDITION());
	YYSETCONDITION(new_state);
}

#define yy_push_state(state_and_tsrm) _yy_push_state(yyc##state_and_tsrm)

static void yy_pop_state(void)
{
	int *stack_state = zend_stack_top(&SCNG(state_stack));
	YYSETCONDITION(*stack_state);
	zend_stack_del_top(&SCNG(state_stack));
}

static void yy_scan_buffer(char *str, unsigned int len)
{
	YYCURSOR = (YYCTYPE*)str;
	SCNG(yy_start) = YYCURSOR;
	YYLIMIT  = YYCURSOR + len;
}

#define ini_filename SCNG(filename)

/* {{{ init_ini_scanner()
*/
static int init_ini_scanner(int scanner_mode, zend_file_handle *fh)
{
	/* Sanity check */
	if (scanner_mode != ZEND_INI_SCANNER_NORMAL && scanner_mode != ZEND_INI_SCANNER_RAW && scanner_mode != ZEND_INI_SCANNER_TYPED) {
		zend_error(E_WARNING, "Invalid scanner mode");
		return FAILURE;
	}

	SCNG(lineno) = 1;
	SCNG(scanner_mode) = scanner_mode;
	SCNG(yy_in) = fh;

	if (fh != NULL) {
		ini_filename = zend_strndup(fh->filename, strlen(fh->filename));
	} else {
		ini_filename = NULL;
	}

	zend_stack_init(&SCNG(state_stack), sizeof(int));
	BEGIN(INITIAL);

	return SUCCESS;
}
/* }}} */

/* {{{ shutdown_ini_scanner()
*/
void shutdown_ini_scanner(void)
{
	zend_stack_destroy(&SCNG(state_stack));
	if (ini_filename) {
		free(ini_filename);
	}
}
/* }}} */

/* {{{ zend_ini_scanner_get_lineno()
*/
int zend_ini_scanner_get_lineno(void)
{
	return SCNG(lineno);
}
/* }}} */

/* {{{ zend_ini_scanner_get_filename()
*/
char *zend_ini_scanner_get_filename(void)
{
	return ini_filename ? ini_filename : "Unknown";
}
/* }}} */

/* {{{ zend_ini_open_file_for_scanning()
*/
int zend_ini_open_file_for_scanning(zend_file_handle *fh, int scanner_mode)
{
	char *buf;
	size_t size;

	if (zend_stream_fixup(fh, &buf, &size) == FAILURE) {
		return FAILURE;
	}

	if (init_ini_scanner(scanner_mode, fh) == FAILURE) {
		zend_file_handle_dtor(fh);
		return FAILURE;
	}

	yy_scan_buffer(buf, (unsigned int)size);

	return SUCCESS;
}
/* }}} */

/* {{{ zend_ini_prepare_string_for_scanning()
*/
int zend_ini_prepare_string_for_scanning(char *str, int scanner_mode)
{
	int len = (int)strlen(str);

	if (init_ini_scanner(scanner_mode, NULL) == FAILURE) {
		return FAILURE;
	}

	yy_scan_buffer(str, len);

	return SUCCESS;
}
/* }}} */

/* {{{ zend_ini_escape_string()
 */
static void zend_ini_escape_string(zval *lval, char *str, int len, char quote_type)
{
	register char *s, *t;
	char *end;

	zend_ini_copy_value(lval, str, len);

	/* convert escape sequences */
	s = t = Z_STRVAL_P(lval);
	end = s + Z_STRLEN_P(lval);

	while (s < end) {
		if (*s == '\\') {
			s++;
			if (s >= end) {
				*t++ = '\\';
				continue;
			}
			switch (*s) {
				case '"':
					if (*s != quote_type) {
						*t++ = '\\';
						*t++ = *s;
						break;
					}
				case '\\':
				case '$':
					*t++ = *s;
					Z_STRLEN_P(lval)--;
					break;
				default:
					*t++ = '\\';
					*t++ = *s;
					break;
			}
		} else {
			*t++ = *s;
		}
		if (*s == '\n' || (*s == '\r' && (*(s+1) != '\n'))) {
			SCNG(lineno)++;
		}
		s++;
	}
	*t = 0;
}
/* }}} */

int ini_lex(zval *ini_lval)
{
restart:
	SCNG(yy_text) = YYCURSOR;

/* yymore_restart: */
	/* detect EOF */
	if (YYCURSOR >= YYLIMIT) {
		if (YYSTATE == STATE(ST_VALUE) || YYSTATE == STATE(ST_RAW)) {
			BEGIN(INITIAL);
			return END_OF_LINE;
		}
		return 0;
	}

	/* Eat any UTF-8 BOM we find in the first 3 bytes */
	if (YYCURSOR == SCNG(yy_start) && YYCURSOR + 3 < YYLIMIT) {
		if (memcmp(YYCURSOR, "\xef\xbb\xbf", 3) == 0) {
			YYCURSOR += 3;
			goto restart;
		}
	}

{
	YYCTYPE yych;
	unsigned int yyaccept = 0;
	static const void* yyctable[8] = {
		&&yyc_INITIAL,
		&&yyc_ST_OFFSET,
		&&yyc_ST_SECTION_VALUE,
		&&yyc_ST_VALUE,
		&&yyc_ST_SECTION_RAW,
		&&yyc_ST_DOUBLE_QUOTES,
		&&yyc_ST_VARNAME,
		&&yyc_ST_RAW
	};
	static const unsigned char yybm_INITIAL[256] = {
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 160,   0, 144, 144,   0, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		240, 128, 128, 144, 128, 144, 128, 144,
		128, 128, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 128, 144, 128, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 128, 144, 144, 128, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 128, 128, 128, 128, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144,
		144, 144, 144, 144, 144, 144, 144, 144
	};
	static const unsigned char yybm_ST_OFFSET[256] = {
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 140, 128, 132, 132, 128, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		140, 132, 128, 132, 192, 132, 132,   0,
		132, 132, 132, 132, 132, 132, 132, 132,
		180, 180, 180, 180, 180, 180, 180, 180,
		180, 180, 132, 128, 132, 132, 132, 132,
		132, 164, 164, 164, 164, 164, 164, 164,
		164, 164, 164, 164, 164, 164, 164, 164,
		164, 164, 164, 164, 164, 164, 164, 164,
		164, 164, 164, 132, 192, 128, 132, 164,
		132, 164, 164, 164, 164, 164, 164, 164,
		164, 164, 164, 164, 164, 164, 164, 164,
		164, 164, 164, 164, 164, 164, 164, 164,
		164, 164, 164, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132
	};
	static const unsigned char yybm_ST_SECTION_VALUE[256] = {
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 140, 128, 132, 132, 128, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		140, 132, 128, 132, 192, 132, 132,   0,
		132, 132, 132, 132, 132, 132, 132, 132,
		180, 180, 180, 180, 180, 180, 180, 180,
		180, 180, 132, 128, 132, 132, 132, 132,
		132, 164, 164, 164, 164, 164, 164, 164,
		164, 164, 164, 164, 164, 164, 164, 164,
		164, 164, 164, 164, 164, 164, 164, 164,
		164, 164, 164, 132, 192, 128, 132, 164,
		132, 164, 164, 164, 164, 164, 164, 164,
		164, 164, 164, 164, 164, 164, 164, 164,
		164, 164, 164, 164, 164, 164, 164, 164,
		164, 164, 164, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132,
		132, 132, 132, 132, 132, 132, 132, 132
	};
	static const unsigned char yybm_ST_VALUE[256] = {
		 80,  82,  82,  82,  82,  82,  82,  82,
		 82,  84,  64,  82,  82,  64,  82,  82,
		 82,  82,  82,  82,  82,  82,  82,  82,
		 82,  82,  82,  82,  82,  82,  82,  82,
		 84,  80,  80,  82, 208,  82,  80,  16,
		 80,  80,  82,  82,  82,  82,  82,  82,
		122, 122, 122, 122, 122, 122, 122, 122,
		122, 122,  82,  80,  82,  80,  82,  82,
		 82, 114, 114, 114, 114, 114, 114, 114,
		114, 114, 114, 114, 114, 114, 114, 114,
		114, 114, 114, 114, 114, 114, 114, 114,
		114, 114, 114,  82,  82,  82,  80, 114,
		 82, 114, 114, 114, 114, 114, 114, 114,
		114, 114, 114, 114, 114, 114, 114, 114,
		114, 114, 114, 114, 114, 114, 114, 114,
		114, 114, 114,  82,  80,  82,  80,  82,
		 82,  82,  82,  82,  82,  82,  82,  82,
		 82,  82,  82,  82,  82,  82,  82,  82,
		 82,  82,  82,  82,  82,  82,  82,  82,
		 82,  82,  82,  82,  82,  82,  82,  82,
		 82,  82,  82,  82,  82,  82,  82,  82,
		 82,  82,  82,  82,  82,  82,  82,  82,
		 82,  82,  82,  82,  82,  82,  82,  82,
		 82,  82,  82,  82,  82,  82,  82,  82,
		 82,  82,  82,  82,  82,  82,  82,  82,
		 82,  82,  82,  82,  82,  82,  82,  82,
		 82,  82,  82,  82,  82,  82,  82,  82,
		 82,  82,  82,  82,  82,  82,  82,  82,
		 82,  82,  82,  82,  82,  82,  82,  82,
		 82,  82,  82,  82,  82,  82,  82,  82,
		 82,  82,  82,  82,  82,  82,  82,  82,
		 82,  82,  82,  82,  82,  82,  82,  82
	};
	static const unsigned char yybm_ST_SECTION_RAW[256] = {
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64, 192,   0,  64,  64,   0,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		192,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,   0,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64
	};
	static const unsigned char yybm_ST_DOUBLE_QUOTES[256] = {
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0, 128,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		128,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0,
		  0,   0,   0,   0,   0,   0,   0,   0
	};
	static const unsigned char yybm_ST_RAW[256] = {
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64, 192,   0,  64,  64,   0,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		192,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64,
		 64,  64,  64,  64,  64,  64,  64,  64
	};
	goto *yyctable[YYGETCONDITION()];
/* *********************************** */
yyc_INITIAL:
	YYFILL(5);
	yych = *YYCURSOR;
	YYDEBUG(0, yych);
	{
		static const void* yytarget[256] = {
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy4,  &&yy6,  &&yy1,  &&yy1,  &&yy8,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy9, &&yy10, &&yy10,  &&yy1, &&yy10,  &&yy1, &&yy10,  &&yy1,
			&&yy10, &&yy10,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1, &&yy11,  &&yy1, &&yy12,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1, &&yy13,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1, &&yy14, &&yy15,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1, &&yy16,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1, &&yy17,  &&yy1, &&yy18,  &&yy1,  &&yy1, &&yy10,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1, &&yy13,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1, &&yy14, &&yy15,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1, &&yy16,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1, &&yy17,  &&yy1, &&yy10, &&yy10, &&yy10, &&yy10,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1
		};
		goto *yytarget[yych];
	}
yy1:
	YYDEBUG(1, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
yy2:
	YYDEBUG(2, yych);
	{
		static const void* yytarget[256] = {
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy3,  &&yy3,  &&yy1,  &&yy1,  &&yy3,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy3,  &&yy3,  &&yy1,  &&yy3,  &&yy1,  &&yy3,  &&yy1,
			 &&yy3,  &&yy3,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy3,  &&yy1,  &&yy3,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1, &&yy19,  &&yy1,  &&yy1,  &&yy3,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy3,  &&yy3,  &&yy3,  &&yy3,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1
		};
		goto *yytarget[yych];
	}
yy3:
	YYDEBUG(3, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Get option name */
	/* Eat leading whitespace */
	EAT_LEADING_WHITESPACE();

	/* Eat trailing whitespace */
	EAT_TRAILING_WHITESPACE();

	RETURN_TOKEN(TC_LABEL, yytext, yyleng);
}
yy4:
	YYDEBUG(4, yych);
	++YYCURSOR;
	YYFILL(2);
	yych = *YYCURSOR;
	{
		static const void* yytarget[256] = {
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy4,  &&yy6,  &&yy5,  &&yy5,  &&yy8,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy4,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5, &&yy11,  &&yy5, &&yy12,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,
			 &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5,  &&yy5
		};
		goto *yytarget[yych];
	}
yy5:
	YYDEBUG(5, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{
	/* eat whitespace */
	goto restart;
}
yy6:
	YYDEBUG(6, yych);
	++YYCURSOR;
yy7:
	YYDEBUG(7, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{
	SCNG(lineno)++;
	return END_OF_LINE;
}
yy8:
	YYDEBUG(8, yych);
	yych = *++YYCURSOR;
	if (yych == '\n') goto yy6;
	goto yy7;
yy9:
	YYDEBUG(9, yych);
	++YYCURSOR;
	YYFILL(2);
	yych = *YYCURSOR;
	{
		static const void* yytarget[256] = {
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy4,  &&yy6,  &&yy1,  &&yy1,  &&yy8,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy9,  &&yy3,  &&yy3,  &&yy1,  &&yy3,  &&yy1,  &&yy3,  &&yy1,
			 &&yy3,  &&yy3,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1, &&yy11,  &&yy1, &&yy12,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1, &&yy19,  &&yy1,  &&yy1,  &&yy3,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy3,  &&yy3,  &&yy3,  &&yy3,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1
		};
		goto *yytarget[yych];
	}
yy10:
	YYDEBUG(10, yych);
	++YYCURSOR;
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Disallow these chars outside option values */
	return yytext[0];
}
yy11:
	YYDEBUG(11, yych);
	++YYCURSOR;
	YYFILL(2);
	yych = *YYCURSOR;
	if (yybm_INITIAL[0+yych] & 128) goto yy11;
	if (yych <= '\n') goto yy20;
	goto yy22;
yy12:
	YYDEBUG(12, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	if (yych == '\t') goto yy12;
	if (yych == ' ') goto yy12;
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Start option value */
	if (SCNG(scanner_mode) == ZEND_INI_SCANNER_RAW) {
		yy_push_state(ST_RAW);
	} else {
		yy_push_state(ST_VALUE);
	}
	return '=';
}
yy13:
	YYDEBUG(13, yych);
	yych = *++YYCURSOR;
	if (yych == 'A') goto yy23;
	if (yych == 'a') goto yy23;
	goto yy2;
yy14:
	YYDEBUG(14, yych);
	yych = *++YYCURSOR;
	{
		static const void* yytarget[256] = {
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2, &&yy24,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2, &&yy26,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2, &&yy24,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2, &&yy26,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2
		};
		goto *yytarget[yych];
	}
yy15:
	YYDEBUG(15, yych);
	yych = *++YYCURSOR;
	{
		static const void* yytarget[256] = {
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2, &&yy27,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2, &&yy28,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2, &&yy27,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2, &&yy28,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2
		};
		goto *yytarget[yych];
	}
yy16:
	YYDEBUG(16, yych);
	yych = *++YYCURSOR;
	if (yych == 'R') goto yy30;
	if (yych == 'r') goto yy30;
	goto yy2;
yy17:
	YYDEBUG(17, yych);
	yych = *++YYCURSOR;
	if (yych == 'E') goto yy31;
	if (yych == 'e') goto yy31;
	goto yy2;
yy18:
	YYDEBUG(18, yych);
	++YYCURSOR;
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Section start */
	/* Enter section data lookup state */
	if (SCNG(scanner_mode) == ZEND_INI_SCANNER_RAW) {
		yy_push_state(ST_SECTION_RAW);
	} else {
		yy_push_state(ST_SECTION_VALUE);
	}
	return TC_SECTION;
}
yy19:
	YYDEBUG(19, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	if (yych == '\t') goto yy19;
	if (yych == ' ') goto yy19;
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Start of option with offset */
	/* Eat leading whitespace */
	EAT_LEADING_WHITESPACE();

	/* Eat trailing whitespace and [ */
	EAT_TRAILING_WHITESPACE_EX('[');

	/* Enter offset lookup state */
	yy_push_state(ST_OFFSET);

	RETURN_TOKEN(TC_OFFSET, yytext, yyleng);
}
yy20:
	YYDEBUG(20, yych);
	++YYCURSOR;
yy21:
	YYDEBUG(21, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Comment */
	BEGIN(INITIAL);
	SCNG(lineno)++;
	return END_OF_LINE;
}
yy22:
	YYDEBUG(22, yych);
	yych = *++YYCURSOR;
	if (yych == '\n') goto yy20;
	goto yy21;
yy23:
	YYDEBUG(23, yych);
	yych = *++YYCURSOR;
	if (yych == 'L') goto yy32;
	if (yych == 'l') goto yy32;
	goto yy2;
yy24:
	YYDEBUG(24, yych);
	yych = *++YYCURSOR;
	{
		static const void* yytarget[256] = {
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2, &&yy33, &&yy25,  &&yy2,  &&yy2, &&yy25,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			&&yy34, &&yy25, &&yy25,  &&yy2, &&yy25,  &&yy2, &&yy25,  &&yy2,
			&&yy25, &&yy25,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2, &&yy25,  &&yy2, &&yy25,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2, &&yy35,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2, &&yy25,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2, &&yy35,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2, &&yy25, &&yy25, &&yy25, &&yy25,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,
			 &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2,  &&yy2
		};
		goto *yytarget[yych];
	}
yy25:
	YYDEBUG(25, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* FALSE value (when used outside option value/offset this causes parse error!)*/
	RETURN_TOKEN(BOOL_FALSE, "", 0);
}
yy26:
	YYDEBUG(26, yych);
	yych = *++YYCURSOR;
	if (yych == 'L') goto yy36;
	if (yych == 'l') goto yy36;
	goto yy2;
yy27:
	YYDEBUG(27, yych);
	yych = *++YYCURSOR;
	if (yych == 'F') goto yy34;
	if (yych == 'f') goto yy34;
	goto yy2;
yy28:
	YYDEBUG(28, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	{
		static const void* yytarget[256] = {
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1, &&yy37, &&yy29,  &&yy1,  &&yy1, &&yy29,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			&&yy28, &&yy29, &&yy29,  &&yy1, &&yy29,  &&yy1, &&yy29,  &&yy1,
			&&yy29, &&yy29,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1, &&yy29,  &&yy1, &&yy29,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1, &&yy19,  &&yy1,  &&yy1, &&yy29,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1, &&yy29, &&yy29, &&yy29, &&yy29,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1
		};
		goto *yytarget[yych];
	}
yy29:
	YYDEBUG(29, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* TRUE value (when used outside option value/offset this causes parse error!) */
	RETURN_TOKEN(BOOL_TRUE, "1", 1);
}
yy30:
	YYDEBUG(30, yych);
	yych = *++YYCURSOR;
	if (yych == 'U') goto yy38;
	if (yych == 'u') goto yy38;
	goto yy2;
yy31:
	YYDEBUG(31, yych);
	yych = *++YYCURSOR;
	if (yych == 'S') goto yy28;
	if (yych == 's') goto yy28;
	goto yy2;
yy32:
	YYDEBUG(32, yych);
	yych = *++YYCURSOR;
	if (yych == 'S') goto yy35;
	if (yych == 's') goto yy35;
	goto yy2;
yy33:
	YYDEBUG(33, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	if (yych == '\t') goto yy33;
	if (yych == ' ') goto yy33;
	goto yy25;
yy34:
	YYDEBUG(34, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	{
		static const void* yytarget[256] = {
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1, &&yy33, &&yy25,  &&yy1,  &&yy1, &&yy25,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			&&yy34, &&yy25, &&yy25,  &&yy1, &&yy25,  &&yy1, &&yy25,  &&yy1,
			&&yy25, &&yy25,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1, &&yy25,  &&yy1, &&yy25,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1, &&yy19,  &&yy1,  &&yy1, &&yy25,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1, &&yy25, &&yy25, &&yy25, &&yy25,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1
		};
		goto *yytarget[yych];
	}
yy35:
	YYDEBUG(35, yych);
	yych = *++YYCURSOR;
	if (yych == 'E') goto yy34;
	if (yych == 'e') goto yy34;
	goto yy2;
yy36:
	YYDEBUG(36, yych);
	yych = *++YYCURSOR;
	if (yych == 'L') goto yy39;
	if (yych == 'l') goto yy39;
	goto yy2;
yy37:
	YYDEBUG(37, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	if (yych == '\t') goto yy37;
	if (yych == ' ') goto yy37;
	goto yy29;
yy38:
	YYDEBUG(38, yych);
	yych = *++YYCURSOR;
	if (yych == 'E') goto yy28;
	if (yych == 'e') goto yy28;
	goto yy2;
yy39:
	YYDEBUG(39, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	{
		static const void* yytarget[256] = {
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1, &&yy41, &&yy40,  &&yy1,  &&yy1, &&yy40,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			&&yy39, &&yy40, &&yy40,  &&yy1, &&yy40,  &&yy1, &&yy40,  &&yy1,
			&&yy40, &&yy40,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1, &&yy40,  &&yy1, &&yy40,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1, &&yy19,  &&yy1,  &&yy1, &&yy40,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1, &&yy40, &&yy40, &&yy40, &&yy40,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,
			 &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1,  &&yy1
		};
		goto *yytarget[yych];
	}
yy40:
	YYDEBUG(40, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{
	RETURN_TOKEN(NULL_NULL, "", 0);
}
yy41:
	YYDEBUG(41, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	if (yych == '\t') goto yy41;
	if (yych == ' ') goto yy41;
	goto yy40;
/* *********************************** */
yyc_ST_OFFSET:
	YYFILL(2);
	yych = *YYCURSOR;
	YYDEBUG(42, yych);
	{
		static const void* yytarget[256] = {
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy46, &&yy47, &&yy43, &&yy43, &&yy47, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy46, &&yy43, &&yy49, &&yy43, &&yy50, &&yy43, &&yy43, &&yy51,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy52, &&yy52, &&yy43,
			&&yy53, &&yy53, &&yy53, &&yy53, &&yy53, &&yy53, &&yy53, &&yy53,
			&&yy53, &&yy53, &&yy43, &&yy47, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55,
			&&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55,
			&&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55,
			&&yy55, &&yy55, &&yy55, &&yy43, &&yy57, &&yy58, &&yy43, &&yy55,
			&&yy43, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55,
			&&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55,
			&&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55,
			&&yy55, &&yy55, &&yy55, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43
		};
		goto *yytarget[yych];
	}
yy43:
	YYDEBUG(43, yych);
	yyaccept = 0;
	YYMARKER = ++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
yy44:
	YYDEBUG(44, yych);
	if (yybm_ST_OFFSET[0+yych] & 4) goto yy43;
	if (yych <= '"') goto yy45;
	if (yych <= '$') goto yy59;
	if (yych <= ';') goto yy45;
	if (yych <= '\\') goto yy57;
yy45:
	YYDEBUG(45, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Get rest as section/offset value */
	RETURN_TOKEN(TC_STRING, yytext, yyleng);
}
yy46:
	YYDEBUG(46, yych);
	yyaccept = 0;
	YYMARKER = ++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	{
		static const void* yytarget[256] = {
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy46, &&yy45, &&yy43, &&yy43, &&yy45, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy46, &&yy43, &&yy49, &&yy43, &&yy59, &&yy43, &&yy43, &&yy45,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy45, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy57, &&yy58, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43
		};
		goto *yytarget[yych];
	}
yy47:
	YYDEBUG(47, yych);
	++YYCURSOR;
yy48:
	YYDEBUG(48, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{
	return 0;
}
yy49:
	YYDEBUG(49, yych);
	++YYCURSOR;
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Double quoted '"' string start */
	yy_push_state(ST_DOUBLE_QUOTES);
	return '"';
}
yy50:
	YYDEBUG(50, yych);
	yych = *++YYCURSOR;
	if (yych <= '\\') {
		if (yych <= 0x00) goto yy48;
		if (yych <= '[') goto yy43;
		goto yy61;
	} else {
		if (yych == '{') goto yy62;
		goto yy43;
	}
yy51:
	YYDEBUG(51, yych);
	yych = *++YYCURSOR;
	if (yybm_ST_OFFSET[0+yych] & 128) goto yy63;
	goto yy48;
yy52:
	YYDEBUG(52, yych);
	yyaccept = 0;
	yych = *(YYMARKER = ++YYCURSOR);
	if (yych <= '/') goto yy44;
	if (yych <= '9') goto yy64;
	goto yy44;
yy53:
	YYDEBUG(53, yych);
	yyaccept = 1;
	YYMARKER = ++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	{
		static const void* yytarget[256] = {
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy54, &&yy43, &&yy43, &&yy54, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy54, &&yy43, &&yy59, &&yy43, &&yy43, &&yy54,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy64, &&yy43,
			&&yy53, &&yy53, &&yy53, &&yy53, &&yy53, &&yy53, &&yy53, &&yy53,
			&&yy53, &&yy53, &&yy43, &&yy54, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy57, &&yy54, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43
		};
		goto *yytarget[yych];
	}
yy54:
	YYDEBUG(54, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Get number option value as string */
	RETURN_TOKEN(TC_NUMBER, yytext, yyleng);
}
yy55:
	YYDEBUG(55, yych);
	yyaccept = 2;
	YYMARKER = ++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	{
		static const void* yytarget[256] = {
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy56, &&yy43, &&yy43, &&yy56, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy56, &&yy43, &&yy59, &&yy43, &&yy43, &&yy56,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55,
			&&yy55, &&yy55, &&yy43, &&yy56, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55,
			&&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55,
			&&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55,
			&&yy55, &&yy55, &&yy55, &&yy43, &&yy57, &&yy56, &&yy43, &&yy55,
			&&yy43, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55,
			&&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55,
			&&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55, &&yy55,
			&&yy55, &&yy55, &&yy55, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43
		};
		goto *yytarget[yych];
	}
yy56:
	YYDEBUG(56, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Get constant option value */
	RETURN_TOKEN(TC_CONSTANT, yytext, yyleng);
}
yy57:
	YYDEBUG(57, yych);
	++YYCURSOR;
	YYFILL(1);
	goto yy43;
yy58:
	YYDEBUG(58, yych);
	++YYCURSOR;
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* End of section or an option offset */
	BEGIN(INITIAL);
	return ']';
}
yy59:
	YYDEBUG(59, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	if (yych <= '\\') {
		if (yych <= 0x00) goto yy60;
		if (yych <= '[') goto yy43;
		goto yy61;
	} else {
		if (yych != '{') goto yy43;
	}
yy60:
	YYDEBUG(60, yych);
	YYCURSOR = YYMARKER;
	if (yyaccept <= 1) {
		if (yyaccept == 0) goto yy45;
		else goto yy54;
	} else {
		goto yy56;
	}
yy61:
	YYDEBUG(61, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	if (yybm_ST_OFFSET[0+yych] & 64) goto yy61;
	goto yy43;
yy62:
	YYDEBUG(62, yych);
	++YYCURSOR;
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Variable start */
	yy_push_state(ST_VARNAME);
	return TC_DOLLAR_CURLY;
}
yy63:
	YYDEBUG(63, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	if (yybm_ST_OFFSET[0+yych] & 128) goto yy63;
	goto yy65;
yy64:
	YYDEBUG(64, yych);
	yyaccept = 1;
	YYMARKER = ++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	{
		static const void* yytarget[256] = {
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy54, &&yy43, &&yy43, &&yy54, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy54, &&yy43, &&yy59, &&yy43, &&yy43, &&yy54,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy64, &&yy64, &&yy64, &&yy64, &&yy64, &&yy64, &&yy64, &&yy64,
			&&yy64, &&yy64, &&yy43, &&yy54, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy57, &&yy54, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43,
			&&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43, &&yy43
		};
		goto *yytarget[yych];
	}
yy65:
	YYDEBUG(65, yych);
	++YYCURSOR;
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Raw string */
	/* Eat leading and trailing single quotes */
	if (yytext[0] == '\'' && yytext[yyleng - 1] == '\'') {
		SCNG(yy_text)++;
		yyleng = yyleng - 2;
	}
	RETURN_TOKEN(TC_RAW, yytext, yyleng);
}
/* *********************************** */
yyc_ST_SECTION_VALUE:
	YYFILL(2);
	yych = *YYCURSOR;
	YYDEBUG(66, yych);
	{
		static const void* yytarget[256] = {
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy70, &&yy71, &&yy67, &&yy67, &&yy71, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy70, &&yy67, &&yy73, &&yy67, &&yy74, &&yy67, &&yy67, &&yy75,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy76, &&yy76, &&yy67,
			&&yy77, &&yy77, &&yy77, &&yy77, &&yy77, &&yy77, &&yy77, &&yy77,
			&&yy77, &&yy77, &&yy67, &&yy71, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79,
			&&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79,
			&&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79,
			&&yy79, &&yy79, &&yy79, &&yy67, &&yy81, &&yy82, &&yy67, &&yy79,
			&&yy67, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79,
			&&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79,
			&&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79,
			&&yy79, &&yy79, &&yy79, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67
		};
		goto *yytarget[yych];
	}
yy67:
	YYDEBUG(67, yych);
	yyaccept = 0;
	YYMARKER = ++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
yy68:
	YYDEBUG(68, yych);
	if (yybm_ST_SECTION_VALUE[0+yych] & 4) goto yy67;
	if (yych <= '"') goto yy69;
	if (yych <= '$') goto yy84;
	if (yych <= ';') goto yy69;
	if (yych <= '\\') goto yy81;
yy69:
	YYDEBUG(69, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Get rest as section/offset value */
	RETURN_TOKEN(TC_STRING, yytext, yyleng);
}
yy70:
	YYDEBUG(70, yych);
	yyaccept = 0;
	YYMARKER = ++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	{
		static const void* yytarget[256] = {
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy70, &&yy69, &&yy67, &&yy67, &&yy69, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy70, &&yy67, &&yy73, &&yy67, &&yy84, &&yy67, &&yy67, &&yy69,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy69, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy81, &&yy69, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67
		};
		goto *yytarget[yych];
	}
yy71:
	YYDEBUG(71, yych);
	++YYCURSOR;
yy72:
	YYDEBUG(72, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{
	return 0;
}
yy73:
	YYDEBUG(73, yych);
	++YYCURSOR;
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Double quoted '"' string start */
	yy_push_state(ST_DOUBLE_QUOTES);
	return '"';
}
yy74:
	YYDEBUG(74, yych);
	yych = *++YYCURSOR;
	if (yych <= '\\') {
		if (yych <= 0x00) goto yy72;
		if (yych <= '[') goto yy67;
		goto yy86;
	} else {
		if (yych == '{') goto yy87;
		goto yy67;
	}
yy75:
	YYDEBUG(75, yych);
	yych = *++YYCURSOR;
	if (yybm_ST_SECTION_VALUE[0+yych] & 128) goto yy88;
	goto yy72;
yy76:
	YYDEBUG(76, yych);
	yyaccept = 0;
	yych = *(YYMARKER = ++YYCURSOR);
	if (yych <= '/') goto yy68;
	if (yych <= '9') goto yy89;
	goto yy68;
yy77:
	YYDEBUG(77, yych);
	yyaccept = 1;
	YYMARKER = ++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	{
		static const void* yytarget[256] = {
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy78, &&yy67, &&yy67, &&yy78, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy78, &&yy67, &&yy84, &&yy67, &&yy67, &&yy78,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy89, &&yy67,
			&&yy77, &&yy77, &&yy77, &&yy77, &&yy77, &&yy77, &&yy77, &&yy77,
			&&yy77, &&yy77, &&yy67, &&yy78, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy81, &&yy78, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67
		};
		goto *yytarget[yych];
	}
yy78:
	YYDEBUG(78, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Get number option value as string */
	RETURN_TOKEN(TC_NUMBER, yytext, yyleng);
}
yy79:
	YYDEBUG(79, yych);
	yyaccept = 2;
	YYMARKER = ++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	{
		static const void* yytarget[256] = {
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy80, &&yy67, &&yy67, &&yy80, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy80, &&yy67, &&yy84, &&yy67, &&yy67, &&yy80,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79,
			&&yy79, &&yy79, &&yy67, &&yy80, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79,
			&&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79,
			&&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79,
			&&yy79, &&yy79, &&yy79, &&yy67, &&yy81, &&yy80, &&yy67, &&yy79,
			&&yy67, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79,
			&&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79,
			&&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79, &&yy79,
			&&yy79, &&yy79, &&yy79, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67
		};
		goto *yytarget[yych];
	}
yy80:
	YYDEBUG(80, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Get constant option value */
	RETURN_TOKEN(TC_CONSTANT, yytext, yyleng);
}
yy81:
	YYDEBUG(81, yych);
	++YYCURSOR;
	YYFILL(1);
	goto yy67;
yy82:
	YYDEBUG(82, yych);
	++YYCURSOR;
	YYFILL(2);
	yych = *YYCURSOR;
	if (yych <= '\f') {
		if (yych <= 0x08) goto yy83;
		if (yych <= '\t') goto yy82;
		if (yych <= '\n') goto yy90;
	} else {
		if (yych <= '\r') goto yy91;
		if (yych == ' ') goto yy82;
	}
yy83:
	YYDEBUG(83, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* End of section */
	BEGIN(INITIAL);
	SCNG(lineno)++;
	return ']';
}
yy84:
	YYDEBUG(84, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	if (yych <= '\\') {
		if (yych <= 0x00) goto yy85;
		if (yych <= '[') goto yy67;
		goto yy86;
	} else {
		if (yych != '{') goto yy67;
	}
yy85:
	YYDEBUG(85, yych);
	YYCURSOR = YYMARKER;
	if (yyaccept <= 1) {
		if (yyaccept == 0) goto yy69;
		else goto yy78;
	} else {
		goto yy80;
	}
yy86:
	YYDEBUG(86, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	if (yybm_ST_SECTION_VALUE[0+yych] & 64) goto yy86;
	goto yy67;
yy87:
	YYDEBUG(87, yych);
	++YYCURSOR;
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Variable start */
	yy_push_state(ST_VARNAME);
	return TC_DOLLAR_CURLY;
}
yy88:
	YYDEBUG(88, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	if (yybm_ST_SECTION_VALUE[0+yych] & 128) goto yy88;
	goto yy92;
yy89:
	YYDEBUG(89, yych);
	yyaccept = 1;
	YYMARKER = ++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	{
		static const void* yytarget[256] = {
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy78, &&yy67, &&yy67, &&yy78, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy78, &&yy67, &&yy84, &&yy67, &&yy67, &&yy78,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy89, &&yy89, &&yy89, &&yy89, &&yy89, &&yy89, &&yy89, &&yy89,
			&&yy89, &&yy89, &&yy67, &&yy78, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy81, &&yy78, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67,
			&&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67, &&yy67
		};
		goto *yytarget[yych];
	}
yy90:
	YYDEBUG(90, yych);
	++YYCURSOR;
	goto yy83;
yy91:
	YYDEBUG(91, yych);
	yych = *++YYCURSOR;
	if (yych == '\n') goto yy90;
	goto yy83;
yy92:
	YYDEBUG(92, yych);
	++YYCURSOR;
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Raw string */
	/* Eat leading and trailing single quotes */
	if (yytext[0] == '\'' && yytext[yyleng - 1] == '\'') {
		SCNG(yy_text)++;
		yyleng = yyleng - 2;
	}
	RETURN_TOKEN(TC_RAW, yytext, yyleng);
}
/* *********************************** */
yyc_ST_VALUE:
	YYFILL(6);
	yych = *YYCURSOR;
	YYDEBUG(93, yych);
	{
		static const void* yytarget[256] = {
			 &&yy94,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy99, &&yy101,  &&yy96,  &&yy96, &&yy103,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy99, &&yy104, &&yy105,  &&yy96, &&yy106,  &&yy96, &&yy104, &&yy107,
			&&yy104, &&yy104,  &&yy96,  &&yy96,  &&yy96, &&yy108, &&yy108,  &&yy96,
			&&yy109, &&yy109, &&yy109, &&yy109, &&yy109, &&yy109, &&yy109, &&yy109,
			&&yy109, &&yy109,  &&yy96, &&yy111,  &&yy96, &&yy112,  &&yy96,  &&yy96,
			 &&yy96, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy115, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy116, &&yy117,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy118, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy119, &&yy113,  &&yy96,  &&yy96,  &&yy96, &&yy104, &&yy113,
			 &&yy96, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy115, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy116, &&yy117,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy118, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy119, &&yy113,  &&yy96, &&yy104,  &&yy96, &&yy104,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96
		};
		goto *yytarget[yych];
	}
yy94:
	YYDEBUG(94, yych);
	++YYCURSOR;
yy95:
	YYDEBUG(95, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* End of option value (if EOF is reached before EOL */
	BEGIN(INITIAL);
	return END_OF_LINE;
}
yy96:
	YYDEBUG(96, yych);
	yyaccept = 0;
	YYMARKER = ++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
yy97:
	YYDEBUG(97, yych);
	{
		static const void* yytarget[256] = {
			 &&yy98,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy98,  &&yy98,  &&yy96,  &&yy96,  &&yy98,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy98,  &&yy98,  &&yy98,  &&yy96, &&yy120,  &&yy96,  &&yy98,  &&yy98,
			 &&yy98,  &&yy98,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy98,  &&yy96,  &&yy98,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy98,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy98,  &&yy96,  &&yy98,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96
		};
		goto *yytarget[yych];
	}
yy98:
	YYDEBUG(98, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Get everything else as option/offset value */
	RETURN_TOKEN(TC_STRING, yytext, yyleng);
}
yy99:
	YYDEBUG(99, yych);
	++YYCURSOR;
	YYFILL(2);
	yych = *YYCURSOR;
	{
		static const void* yytarget[256] = {
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100,  &&yy99, &&yy101, &&yy100, &&yy100, &&yy103, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			 &&yy99, &&yy100, &&yy105, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy111, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100,
			&&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100, &&yy100
		};
		goto *yytarget[yych];
	}
yy100:
	YYDEBUG(100, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{
	RETURN_TOKEN(TC_WHITESPACE, yytext, yyleng);
}
yy101:
	YYDEBUG(101, yych);
	++YYCURSOR;
yy102:
	YYDEBUG(102, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* End of option value */
	BEGIN(INITIAL);
	SCNG(lineno)++;
	return END_OF_LINE;
}
yy103:
	YYDEBUG(103, yych);
	yych = *++YYCURSOR;
	if (yych == '\n') goto yy101;
	goto yy102;
yy104:
	YYDEBUG(104, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	if (yych == '\t') goto yy104;
	if (yych == ' ') goto yy104;
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Boolean operators */
	return yytext[0];
}
yy105:
	YYDEBUG(105, yych);
	++YYCURSOR;
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Double quoted '"' string start */
	yy_push_state(ST_DOUBLE_QUOTES);
	return '"';
}
yy106:
	YYDEBUG(106, yych);
	yych = *++YYCURSOR;
	if (yych <= '\\') {
		if (yych <= 0x00) goto yy95;
		if (yych <= '[') goto yy96;
		goto yy122;
	} else {
		if (yych == '{') goto yy123;
		goto yy96;
	}
yy107:
	YYDEBUG(107, yych);
	yych = *++YYCURSOR;
	if (yybm_ST_VALUE[0+yych] & 64) goto yy124;
	goto yy95;
yy108:
	YYDEBUG(108, yych);
	yyaccept = 0;
	yych = *(YYMARKER = ++YYCURSOR);
	if (yych <= '/') goto yy97;
	if (yych <= '9') goto yy125;
	goto yy97;
yy109:
	YYDEBUG(109, yych);
	yyaccept = 1;
	YYMARKER = ++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	{
		static const void* yytarget[256] = {
			&&yy110,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96, &&yy110, &&yy110,  &&yy96,  &&yy96, &&yy110,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			&&yy110, &&yy110, &&yy110,  &&yy96, &&yy120,  &&yy96, &&yy110, &&yy110,
			&&yy110, &&yy110,  &&yy96,  &&yy96,  &&yy96,  &&yy96, &&yy125,  &&yy96,
			&&yy109, &&yy109, &&yy109, &&yy109, &&yy109, &&yy109, &&yy109, &&yy109,
			&&yy109, &&yy109,  &&yy96, &&yy110,  &&yy96, &&yy110,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96, &&yy110,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96, &&yy110,  &&yy96, &&yy110,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96
		};
		goto *yytarget[yych];
	}
yy110:
	YYDEBUG(110, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Get number option value as string */
	RETURN_TOKEN(TC_NUMBER, yytext, yyleng);
}
yy111:
	YYDEBUG(111, yych);
	++YYCURSOR;
	YYFILL(2);
	yych = *YYCURSOR;
	if (yybm_ST_VALUE[0+yych] & 16) goto yy111;
	if (yych <= '\n') goto yy126;
	goto yy128;
yy112:
	YYDEBUG(112, yych);
	++YYCURSOR;
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Make = used in option value to trigger error */
	yyless(0);
	BEGIN(INITIAL);
	return END_OF_LINE;
}
yy113:
	YYDEBUG(113, yych);
	yyaccept = 2;
	YYMARKER = ++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	{
		static const void* yytarget[256] = {
			&&yy114,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96, &&yy114, &&yy114,  &&yy96,  &&yy96, &&yy114,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			&&yy114, &&yy114, &&yy114,  &&yy96, &&yy120,  &&yy96, &&yy114, &&yy114,
			&&yy114, &&yy114,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113,  &&yy96, &&yy114,  &&yy96, &&yy114,  &&yy96,  &&yy96,
			 &&yy96, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy96,  &&yy96,  &&yy96, &&yy114, &&yy113,
			 &&yy96, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy96, &&yy114,  &&yy96, &&yy114,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96
		};
		goto *yytarget[yych];
	}
yy114:
	YYDEBUG(114, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Get constant option value */
	RETURN_TOKEN(TC_CONSTANT, yytext, yyleng);
}
yy115:
	YYDEBUG(115, yych);
	yyaccept = 2;
	yych = *(YYMARKER = ++YYCURSOR);
	{
		static const void* yytarget[256] = {
			&&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97, &&yy114, &&yy114,  &&yy97,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy114, &&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy114,
			&&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97, &&yy129, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy113,
			 &&yy97, &&yy129, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97
		};
		goto *yytarget[yych];
	}
yy116:
	YYDEBUG(116, yych);
	yyaccept = 2;
	yych = *(YYMARKER = ++YYCURSOR);
	{
		static const void* yytarget[256] = {
			&&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97, &&yy114, &&yy114,  &&yy97,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy114, &&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy114,
			&&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy130,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy132, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy113,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy130,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy132, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97
		};
		goto *yytarget[yych];
	}
yy117:
	YYDEBUG(117, yych);
	yyaccept = 2;
	yych = *(YYMARKER = ++YYCURSOR);
	{
		static const void* yytarget[256] = {
			&&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97, &&yy114, &&yy114,  &&yy97,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy114, &&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy114,
			&&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy133, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy134, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy113,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy133, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy134, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97
		};
		goto *yytarget[yych];
	}
yy118:
	YYDEBUG(118, yych);
	yyaccept = 2;
	yych = *(YYMARKER = ++YYCURSOR);
	{
		static const void* yytarget[256] = {
			&&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97, &&yy114, &&yy114,  &&yy97,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy114, &&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy114,
			&&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy136, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy113,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy136, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97
		};
		goto *yytarget[yych];
	}
yy119:
	YYDEBUG(119, yych);
	yyaccept = 2;
	yych = *(YYMARKER = ++YYCURSOR);
	{
		static const void* yytarget[256] = {
			&&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97, &&yy114, &&yy114,  &&yy97,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy114, &&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy114,
			&&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy137, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy113,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy137, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97
		};
		goto *yytarget[yych];
	}
yy120:
	YYDEBUG(120, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	if (yych <= '\\') {
		if (yych <= 0x00) goto yy121;
		if (yych <= '[') goto yy96;
		goto yy122;
	} else {
		if (yych != '{') goto yy96;
	}
yy121:
	YYDEBUG(121, yych);
	YYCURSOR = YYMARKER;
	if (yyaccept <= 2) {
		if (yyaccept <= 1) {
			if (yyaccept == 0) goto yy98;
			else goto yy110;
		} else {
			goto yy114;
		}
	} else {
		if (yyaccept <= 4) {
			if (yyaccept == 3) goto yy131;
			else goto yy135;
		} else {
			goto yy148;
		}
	}
yy122:
	YYDEBUG(122, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	if (yybm_ST_VALUE[0+yych] & 128) goto yy138;
	goto yy96;
yy123:
	YYDEBUG(123, yych);
	++YYCURSOR;
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Variable start */
	yy_push_state(ST_VARNAME);
	return TC_DOLLAR_CURLY;
}
yy124:
	YYDEBUG(124, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	if (yybm_ST_VALUE[0+yych] & 64) goto yy124;
	goto yy139;
yy125:
	YYDEBUG(125, yych);
	yyaccept = 1;
	YYMARKER = ++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	{
		static const void* yytarget[256] = {
			&&yy110,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96, &&yy110, &&yy110,  &&yy96,  &&yy96, &&yy110,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			&&yy110, &&yy110, &&yy110,  &&yy96, &&yy120,  &&yy96, &&yy110, &&yy110,
			&&yy110, &&yy110,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			&&yy125, &&yy125, &&yy125, &&yy125, &&yy125, &&yy125, &&yy125, &&yy125,
			&&yy125, &&yy125,  &&yy96, &&yy110,  &&yy96, &&yy110,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96, &&yy110,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96, &&yy110,  &&yy96, &&yy110,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,
			 &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96,  &&yy96
		};
		goto *yytarget[yych];
	}
yy126:
	YYDEBUG(126, yych);
	++YYCURSOR;
yy127:
	YYDEBUG(127, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Comment */
	BEGIN(INITIAL);
	SCNG(lineno)++;
	return END_OF_LINE;
}
yy128:
	YYDEBUG(128, yych);
	yych = *++YYCURSOR;
	if (yych == '\n') goto yy126;
	goto yy127;
yy129:
	YYDEBUG(129, yych);
	yyaccept = 2;
	yych = *(YYMARKER = ++YYCURSOR);
	{
		static const void* yytarget[256] = {
			&&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97, &&yy114, &&yy114,  &&yy97,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy114, &&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy114,
			&&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy140, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy113,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy140, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97
		};
		goto *yytarget[yych];
	}
yy130:
	YYDEBUG(130, yych);
	yyaccept = 3;
	yych = *(YYMARKER = ++YYCURSOR);
	{
		static const void* yytarget[256] = {
			&&yy131,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97, &&yy141, &&yy131,  &&yy97,  &&yy97, &&yy131,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy141, &&yy131, &&yy131,  &&yy97,  &&yy97,  &&yy97, &&yy131, &&yy131,
			&&yy131, &&yy131,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113,  &&yy97, &&yy131,  &&yy97, &&yy131,  &&yy97,  &&yy97,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy142, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97,  &&yy97,  &&yy97, &&yy131, &&yy113,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy142, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97, &&yy131,  &&yy97, &&yy131,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97
		};
		goto *yytarget[yych];
	}
yy131:
	YYDEBUG(131, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* FALSE value (when used outside option value/offset this causes parse error!)*/
	RETURN_TOKEN(BOOL_FALSE, "", 0);
}
yy132:
	YYDEBUG(132, yych);
	yyaccept = 2;
	yych = *(YYMARKER = ++YYCURSOR);
	{
		static const void* yytarget[256] = {
			&&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97, &&yy114, &&yy114,  &&yy97,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy114, &&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy114,
			&&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy143, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy113,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy143, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97
		};
		goto *yytarget[yych];
	}
yy133:
	YYDEBUG(133, yych);
	yyaccept = 2;
	yych = *(YYMARKER = ++YYCURSOR);
	{
		static const void* yytarget[256] = {
			&&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97, &&yy114, &&yy114,  &&yy97,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy114, &&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy114,
			&&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy144, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy113,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy144, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97
		};
		goto *yytarget[yych];
	}
yy134:
	YYDEBUG(134, yych);
	yyaccept = 4;
	yych = *(YYMARKER = ++YYCURSOR);
	{
		static const void* yytarget[256] = {
			&&yy135,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97, &&yy145, &&yy135,  &&yy97,  &&yy97, &&yy135,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy145, &&yy135, &&yy135,  &&yy97,  &&yy97,  &&yy97, &&yy135, &&yy135,
			&&yy135, &&yy135,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113,  &&yy97, &&yy135,  &&yy97, &&yy135,  &&yy97,  &&yy97,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97,  &&yy97,  &&yy97, &&yy135, &&yy113,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97, &&yy135,  &&yy97, &&yy135,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97
		};
		goto *yytarget[yych];
	}
yy135:
	YYDEBUG(135, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* TRUE value (when used outside option value/offset this causes parse error!) */
	RETURN_TOKEN(BOOL_TRUE, "1", 1);
}
yy136:
	YYDEBUG(136, yych);
	yyaccept = 2;
	yych = *(YYMARKER = ++YYCURSOR);
	{
		static const void* yytarget[256] = {
			&&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97, &&yy114, &&yy114,  &&yy97,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy114, &&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy114,
			&&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy146, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy113,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy146, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97
		};
		goto *yytarget[yych];
	}
yy137:
	YYDEBUG(137, yych);
	yyaccept = 2;
	yych = *(YYMARKER = ++YYCURSOR);
	{
		static const void* yytarget[256] = {
			&&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97, &&yy114, &&yy114,  &&yy97,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy114, &&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy114,
			&&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy134, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy113,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy134, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97
		};
		goto *yytarget[yych];
	}
yy138:
	YYDEBUG(138, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	if (yybm_ST_VALUE[0+yych] & 128) goto yy138;
	if (yych <= 0x00) goto yy98;
	if (yych == '\\') goto yy122;
	goto yy96;
yy139:
	YYDEBUG(139, yych);
	++YYCURSOR;
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Raw string */
	/* Eat leading and trailing single quotes */
	if (yytext[0] == '\'' && yytext[yyleng - 1] == '\'') {
		SCNG(yy_text)++;
		yyleng = yyleng - 2;
	}
	RETURN_TOKEN(TC_RAW, yytext, yyleng);
}
yy140:
	YYDEBUG(140, yych);
	yyaccept = 2;
	yych = *(YYMARKER = ++YYCURSOR);
	{
		static const void* yytarget[256] = {
			&&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97, &&yy114, &&yy114,  &&yy97,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy114, &&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy114,
			&&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy142, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy113,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy142, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97
		};
		goto *yytarget[yych];
	}
yy141:
	YYDEBUG(141, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	if (yych == '\t') goto yy141;
	if (yych == ' ') goto yy141;
	goto yy131;
yy142:
	YYDEBUG(142, yych);
	yyaccept = 2;
	yych = *(YYMARKER = ++YYCURSOR);
	{
		static const void* yytarget[256] = {
			&&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97, &&yy114, &&yy114,  &&yy97,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy114, &&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy114,
			&&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy144, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy113,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy144, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97
		};
		goto *yytarget[yych];
	}
yy143:
	YYDEBUG(143, yych);
	yyaccept = 2;
	yych = *(YYMARKER = ++YYCURSOR);
	{
		static const void* yytarget[256] = {
			&&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97, &&yy114, &&yy114,  &&yy97,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy114, &&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy114,
			&&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy147, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy113,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy147, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97
		};
		goto *yytarget[yych];
	}
yy144:
	YYDEBUG(144, yych);
	yyaccept = 3;
	yych = *(YYMARKER = ++YYCURSOR);
	{
		static const void* yytarget[256] = {
			&&yy131,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97, &&yy141, &&yy131,  &&yy97,  &&yy97, &&yy131,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy141, &&yy131, &&yy131,  &&yy97,  &&yy97,  &&yy97, &&yy131, &&yy131,
			&&yy131, &&yy131,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113,  &&yy97, &&yy131,  &&yy97, &&yy131,  &&yy97,  &&yy97,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97,  &&yy97,  &&yy97, &&yy131, &&yy113,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97, &&yy131,  &&yy97, &&yy131,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97
		};
		goto *yytarget[yych];
	}
yy145:
	YYDEBUG(145, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	if (yych == '\t') goto yy145;
	if (yych == ' ') goto yy145;
	goto yy135;
yy146:
	YYDEBUG(146, yych);
	yyaccept = 2;
	yych = *(YYMARKER = ++YYCURSOR);
	{
		static const void* yytarget[256] = {
			&&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97, &&yy114, &&yy114,  &&yy97,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy114, &&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy114,
			&&yy114, &&yy114,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,  &&yy97,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy134, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97,  &&yy97,  &&yy97, &&yy114, &&yy113,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy134, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97, &&yy114,  &&yy97, &&yy114,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97
		};
		goto *yytarget[yych];
	}
yy147:
	YYDEBUG(147, yych);
	yyaccept = 5;
	yych = *(YYMARKER = ++YYCURSOR);
	{
		static const void* yytarget[256] = {
			&&yy148,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97, &&yy149, &&yy148,  &&yy97,  &&yy97, &&yy148,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy149, &&yy148, &&yy148,  &&yy97,  &&yy97,  &&yy97, &&yy148, &&yy148,
			&&yy148, &&yy148,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113,  &&yy97, &&yy148,  &&yy97, &&yy148,  &&yy97,  &&yy97,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97,  &&yy97,  &&yy97, &&yy148, &&yy113,
			 &&yy97, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113, &&yy113,
			&&yy113, &&yy113, &&yy113,  &&yy97, &&yy148,  &&yy97, &&yy148,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,
			 &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97,  &&yy97
		};
		goto *yytarget[yych];
	}
yy148:
	YYDEBUG(148, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{
	RETURN_TOKEN(NULL_NULL, "", 0);
}
yy149:
	YYDEBUG(149, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	if (yych == '\t') goto yy149;
	if (yych == ' ') goto yy149;
	goto yy148;
/* *********************************** */
yyc_ST_SECTION_RAW:
	YYFILL(1);
	yych = *YYCURSOR;
	YYDEBUG(150, yych);
	if (yybm_ST_SECTION_RAW[0+yych] & 64) goto yy151;
	if (yych <= '\r') goto yy152;
	goto yy153;
yy151:
	YYDEBUG(151, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	if (yybm_ST_SECTION_RAW[0+yych] & 64) goto yy151;
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Raw value, only used when SCNG(scanner_mode) == ZEND_INI_SCANNER_RAW. */
	RETURN_TOKEN(TC_RAW, yytext, yyleng);
}
yy152:
	YYDEBUG(152, yych);
	++YYCURSOR;
	yyleng = YYCURSOR - SCNG(yy_text);
	{
	return 0;
}
yy153:
	YYDEBUG(153, yych);
	++YYCURSOR;
	YYFILL(2);
	yych = *YYCURSOR;
	if (yybm_ST_SECTION_RAW[0+yych] & 128) goto yy153;
	if (yych <= 0x08) goto yy154;
	if (yych <= '\n') goto yy155;
	if (yych == '\r') goto yy156;
yy154:
	YYDEBUG(154, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* End of section */
	BEGIN(INITIAL);
	SCNG(lineno)++;
	return ']';
}
yy155:
	YYDEBUG(155, yych);
	++YYCURSOR;
	goto yy154;
yy156:
	YYDEBUG(156, yych);
	yych = *++YYCURSOR;
	if (yych == '\n') goto yy155;
	goto yy154;
/* *********************************** */
yyc_ST_DOUBLE_QUOTES:
	YYFILL(2);
	yych = *YYCURSOR;
	YYDEBUG(157, yych);
	if (yych == '"') goto yy159;
	if (yych == '$') goto yy160;
	++YYCURSOR;
yy158:
	YYDEBUG(158, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Escape double quoted string contents */
	if (YYCURSOR > YYLIMIT) {
		return 0;
	}

	while (YYCURSOR < YYLIMIT) {
		switch (*YYCURSOR++) {
			case '"':
				if (YYCURSOR < YYLIMIT && YYCURSOR[-2] == '\\' && *YYCURSOR != '\r' && *YYCURSOR != '\n') {
					continue;
				}
				break;
			case '$':
				if (*YYCURSOR == '{') {
					break;
				}
				continue;
			case '\\':
				if (YYCURSOR < YYLIMIT && *YYCURSOR != '"') {
					YYCURSOR++;
				}
				/* fall through */
			default:
				continue;
		}

		YYCURSOR--;
		break;
	}

	yyleng = YYCURSOR - SCNG(yy_text);

	zend_ini_escape_string(ini_lval, yytext, yyleng, '"');
	return TC_QUOTED_STRING;
}
yy159:
	YYDEBUG(159, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	if (yybm_ST_DOUBLE_QUOTES[0+yych] & 128) goto yy159;
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Double quoted '"' string ends */
	yy_pop_state();
	return '"';
}
yy160:
	YYDEBUG(160, yych);
	yych = *++YYCURSOR;
	if (yych != '{') goto yy158;
	++YYCURSOR;
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Variable start */
	yy_push_state(ST_VARNAME);
	return TC_DOLLAR_CURLY;
}
/* *********************************** */
yyc_ST_VARNAME:
	YYFILL(1);
	yych = *YYCURSOR;
	YYDEBUG(161, yych);
	{
		static const void* yytarget[256] = {
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy164, &&yy164, &&yy162, &&yy162, &&yy164, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy164, &&yy164, &&yy162, &&yy164, &&yy162, &&yy164, &&yy162,
			&&yy164, &&yy164, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy164, &&yy162, &&yy164, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy164, &&yy162, &&yy162, &&yy164, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy164, &&yy164, &&yy165, &&yy164, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162
		};
		goto *yytarget[yych];
	}
yy162:
	YYDEBUG(162, yych);
	++YYCURSOR;
	YYFILL(1);
	yych = *YYCURSOR;
	{
		static const void* yytarget[256] = {
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy163, &&yy163, &&yy162, &&yy162, &&yy163, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy163, &&yy163, &&yy162, &&yy163, &&yy162, &&yy163, &&yy162,
			&&yy163, &&yy163, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy163, &&yy162, &&yy163, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy163, &&yy162, &&yy162, &&yy163, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy163, &&yy163, &&yy163, &&yy163, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162,
			&&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162, &&yy162
		};
		goto *yytarget[yych];
	}
yy163:
	YYDEBUG(163, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Variable name */
	/* Eat leading whitespace */
	EAT_LEADING_WHITESPACE();

	/* Eat trailing whitespace */
	EAT_TRAILING_WHITESPACE();

	RETURN_TOKEN(TC_VARNAME, yytext, yyleng);
}
yy164:
	YYDEBUG(164, yych);
	++YYCURSOR;
	yyleng = YYCURSOR - SCNG(yy_text);
	{
	return 0;
}
yy165:
	YYDEBUG(165, yych);
	++YYCURSOR;
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Variable end */
	yy_pop_state();
	return '}';
}
/* *********************************** */
yyc_ST_RAW:
	YYFILL(3);
	yych = *YYCURSOR;
	YYDEBUG(166, yych);
	{
		static const void* yytarget[256] = {
			&&yy167, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy170, &&yy171, &&yy168, &&yy168, &&yy173, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy170, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy174, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168,
			&&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168, &&yy168
		};
		goto *yytarget[yych];
	}
yy167:
	YYDEBUG(167, yych);
	++YYCURSOR;
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* End of option value (if EOF is reached before EOL */
	BEGIN(INITIAL);
	return END_OF_LINE;
}
yy168:
	YYDEBUG(168, yych);
	++YYCURSOR;
yy169:
	YYDEBUG(169, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Raw value, only used when SCNG(scanner_mode) == ZEND_INI_SCANNER_RAW. */
	unsigned char *sc = NULL;
	while (YYCURSOR < YYLIMIT) {
		switch (*YYCURSOR) {
			case '\n':
			case '\r':
				goto end_raw_value_chars;
				break;
			case ';':
				if (sc == NULL) {
					sc = YYCURSOR;
				}
				/* no break */
			default:
				YYCURSOR++;
				break;
		}
	}
end_raw_value_chars:
	yyleng = YYCURSOR - SCNG(yy_text);

	/* Eat trailing semicolons */
	while (yytext[yyleng - 1] == ';') {
		yyleng--;
	}

	/* Eat leading and trailing double quotes */
	if (yytext[0] == '"' && yytext[yyleng - 1] == '"') {
		SCNG(yy_text)++;
		yyleng = yyleng - 2;
	} else if (sc) {
		YYCURSOR = sc;
		yyleng = YYCURSOR - SCNG(yy_text);
	}
	RETURN_TOKEN(TC_RAW, yytext, yyleng);
}
yy170:
	YYDEBUG(170, yych);
	yych = *++YYCURSOR;
	{
		static const void* yytarget[256] = {
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy176, &&yy176, &&yy169, &&yy169, &&yy176, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy176, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy176, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169,
			&&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169, &&yy169
		};
		goto *yytarget[yych];
	}
yy171:
	YYDEBUG(171, yych);
	++YYCURSOR;
yy172:
	YYDEBUG(172, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* End of option value */
	BEGIN(INITIAL);
	SCNG(lineno)++;
	return END_OF_LINE;
}
yy173:
	YYDEBUG(173, yych);
	yych = *++YYCURSOR;
	if (yych == '\n') goto yy171;
	goto yy172;
yy174:
	YYDEBUG(174, yych);
	++YYCURSOR;
	YYFILL(2);
	yych = *YYCURSOR;
	if (yybm_ST_RAW[0+yych] & 64) goto yy174;
	if (yych <= '\n') goto yy178;
	goto yy180;
yy175:
	YYDEBUG(175, yych);
	++YYCURSOR;
	YYFILL(2);
	yych = *YYCURSOR;
yy176:
	YYDEBUG(176, yych);
	if (yybm_ST_RAW[0+yych] & 128) goto yy175;
	if (yych <= '\f') {
		if (yych <= 0x08) goto yy177;
		if (yych <= '\n') goto yy171;
	} else {
		if (yych <= '\r') goto yy173;
		if (yych == ';') goto yy174;
	}
yy177:
	YYDEBUG(177, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{
	/* eat whitespace */
	goto restart;
}
yy178:
	YYDEBUG(178, yych);
	++YYCURSOR;
yy179:
	YYDEBUG(179, yych);
	yyleng = YYCURSOR - SCNG(yy_text);
	{ /* Comment */
	BEGIN(INITIAL);
	SCNG(lineno)++;
	return END_OF_LINE;
}
yy180:
	YYDEBUG(180, yych);
	yych = *++YYCURSOR;
	if (yych == '\n') goto yy178;
	goto yy179;
}

}
real_world/php20150211_zend_ini_scanner.re:385:13: warning: escape has no effect: '\.' [-Wuseless-escape]
real_world/php20150211_zend_ini_scanner.re:385:32: warning: escape has no effect: '\.' [-Wuseless-escape]
real_world/php20150211_zend_ini_scanner.re:392:27: warning: escape has no effect: '\[' [-Wuseless-escape]
real_world/php20150211_zend_ini_scanner.re:393:11: warning: escape has no effect: '\[' [-Wuseless-escape]
real_world/php20150211_zend_ini_scanner.re:649:7: warning: unreachable rule in condition 'INITIAL' (shadowed by rules at lines 407, 482, 492, 556, 628, 633, 638) [-Wunreachable-rules]
real_world/php20150211_zend_ini_scanner.re:624:50: warning: unreachable rule in condition 'ST_OFFSET' (shadowed by rule at line 574) [-Wunreachable-rules]
real_world/php20150211_zend_ini_scanner.re:624:50: warning: unreachable rule in condition 'ST_SECTION_VALUE' (shadowed by rule at line 574) [-Wunreachable-rules]
real_world/php20150211_zend_ini_scanner.re:649:7: warning: unreachable rule in condition 'ST_VALUE' (shadowed by rules at lines 542, 548, 552, 560, 564, 570, 578, 624, 644) [-Wunreachable-rules]
real_world/php20150211_zend_ini_scanner.re:649:7: warning: unreachable rule in condition 'ST_DOUBLE_QUOTES' (shadowed by rules at lines 583, 588) [-Wunreachable-rules]
real_world/php20150211_zend_ini_scanner.re:649:7: warning: unreachable rule in condition 'ST_RAW' (shadowed by rules at lines 501, 542, 644) [-Wunreachable-rules]
real_world/php20150211_zend_ini_scanner.re:382:0: warning: condition numbers may change, use `conditions` block to generate reliable condition identifiers [-Wcondition-order]
